/*****************************************************************************
 *                Copyright (C) 2013 Radian Memory Systems
 *
 * This file is provided under non-disclosure agreement.
 * See the file COPYING for details.
 ******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if 0
#include <stdtypes.h>
#endif
#include <linux/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <getopt.h>
#include "nvme_express.h"
#include "nvme_ioctl.h"
#if 1
typedef __u64 u64;
typedef __u32 u32;
typedef __u16 u16;
typedef __u8 u8;
#endif
#define MAX_DIES 256
#include "RMSVendorSpecific.h"
#include "nvme_util.h"

#define CONTROL_DEV		"/dev/nvme0"

static void
create_rms200_flush_ctl(
	struct usr_io *x,
	int disable)
{
	nvme_cmd_rms200_t nvme_cmd_rms200;

	create_generic_uio_pkt(x, XFER_FROM_DEV, (void *) 0, 0);

	memset(&nvme_cmd_rms200, 0x0, sizeof(nvme_cmd_rms200));

	if (disable)
		nvme_cmd_rms200.subOpcode = RMS_CLEAN_SHDN_NVRAM_ERASED;
	else
		nvme_cmd_rms200.subOpcode = RMS_CLEAN_SHDN_NVRAM_SAVED;

	append_rms_cmd_pkt(x, &nvme_cmd_rms200);
}

static void
wait_for_q(void)
{
	system("/bin/stty raw");
	getchar();
	system("/bin/stty cooked");
}

static void
description(void)
{
	printf("Restore the RMS_200 memory from Flash.\n");
}

static void
usage(void)
{
	printf("usage:  flush_ctl [-u <devName>] -e | -d\n");
	printf("   -u <devName>: The device to control - default /dev/nvme0\n");
	printf("   -e: Enable flush on shutdown\n");
	printf("   -d: Disable flush on shutdown\n");
}

int
main(int argc, char **argv)
{
	struct usr_io x;
	int status;
	int fd;
	int disable = 2;
	char *control_dev = CONTROL_DEV;

	while (1) {
		int c;

		c = getopt(argc, argv, "hHedu:");
		if (c == -1)
			break;

		switch (c) {

		case 'H':
			description();
			usage();
			wait_for_q();
			exit(1);
			break;

		case 'e':
			disable = 0;
			break;

		case 'd':
			disable = 1;
			break;

		case 'u':
			control_dev = optarg;
			break;

		case 'h':
		default:
			goto err_exit;
		}
	}

	if (disable == 2)
		goto err_exit;

	fd = open(control_dev, O_RDWR);
	if (fd < 0) {
		printf("Could not open file %s ", control_dev);
		if (errno == EACCES)
			printf(" Access error - try running as su\n");
		else
			printf(" Error %d\n", errno);
		exit(1);
	}
	create_rms200_flush_ctl(&x, disable);
	status = ioctl(fd, NVME_IOCTL_ADMIN_CMD, &x);
	if (status != 0) {
		printf("status = %d\n", status);
		goto done;
	}
	if (x.comp.SCT != 0 || x.comp.SC != 0) {
		printf("Operation failed, NVMe error = %s\n",
				nvme_err(x.comp.SCT, x.comp.SC));
		goto done;
	}
done:
	if (fd > 0)
		close(fd);
	exit(0);
err_exit:
	usage();
	exit(1);
}
