/*
 * Copyright 2013-2018 Radian Memory Systems Inc.  All Rights Reserved
 *
 */
#ifndef NVME_EXPRESS_H
#define NVME_EXPRESS_H

#include <stdint.h>

#define NVME_VNDR_CMD_SYS_DEBUG     0xC5

typedef enum {
    NVM_ADMIN_CMD_GET_LOG_PAGE      = 0x02,
    NVM_ADMIN_CMD_FIRMWARE_ACTIVATE = 0x10,
    NVM_ADMIN_CMD_FIRMWARE_DOWNLOAD = 0x11,
} nvm_admin_opcodes_e;

#define GLP_ID_ERR_INFO                0x1  // 64 bytes
#define GLP_ID_SMART_HEALTH            0x2  // 512 bytes
#define GLP_ID_FIRMWARE_SLOT_INFO      0x3  // 512 bytes

struct error_log {
    union {
        struct {
            __u64 errorCount;
            uint16_t sqID;
            uint16_t cmdID;
            uint16_t status;
            uint16_t errorByte:8, errorBit:3, reservedA:5;
            __u64 lba;
            uint32_t nameSpace;
            uint8_t vendorInfo;
            uint8_t reservedB[63-29+1];
        };
        uint32_t   asUlong[16];
    };
};

struct smart_log {
    union {
        struct __attribute__((__packed__)) {
            uint32_t criticalError:8;
            uint32_t temperature:16;
            uint32_t availableSpace:8;
            uint8_t availableSpaceThreshold;
            uint8_t precentageUsed;
            uint8_t reservedA[31-6+1];
            uint8_t dataUnitsRead[47-32+1];
            uint8_t dataUnitsWritten[63-48+1];
            uint8_t hostReadCommands[79-64+1];
            uint8_t hostWriteCommands[95-80+1];
            uint8_t controllerBusyTime[111-96+1];
            uint8_t powerCycles[127-112+1];
            uint8_t powerOnHours[143-128+1];
            uint8_t unsafeShutdowns[159-144+1];
            uint8_t mediaErrors[175-160+1];
            uint8_t numberOfErrorInfoLogs[191-176+1];
            uint8_t reservedB[511-192+1];
        };
        uint8_t   asByte[512];
    };
};

struct firmware_slot_log {
    union {
        struct {
            uint8_t activeFirmwareInfo;
            uint8_t reservedA[7-1+1];
            uint8_t FirmwareRevisionSlot1[15-8+1];
            uint8_t FirmwareRevisionSlot2[23-16+1];
            uint8_t FirmwareRevisionSlot3[31-24+1];
            uint8_t FirmwareRevisionSlot4[39-32+1];
            uint8_t FirmwareRevisionSlot5[47-40+1];
            uint8_t FirmwareRevisionSlot6[55-48+1];
            uint8_t FirmwareRevisionSlot7[63-56+1];
            uint8_t reservedB[511-64+1];
        };
        uint8_t   asByte[512];
    };
};

struct nvme_prp {
    union {
        struct {
            uint32_t lower;
            uint32_t upper;
        };
        uint64_t addr;
    };
};

struct nvme_cmd_hdr {
    uint32_t opCode:8,
        fusedOp:2,
        :6,
        cmdID:16;
    uint32_t namespaceID;
    uint64_t reserved;
    uint64_t metadataPtr;
    struct nvme_prp prp[2];
};

struct nvme_cmd_get_log_page {
    uint16_t     LogPageID;
    uint16_t     numDW:12,reserved:4;
};

struct nvme_cmd_firmware_activate {
    uint32_t     slot: 3,
            action: 2,    : 24;
};

struct nvme_cmd_firmware_download {
    uint32_t     numDW;
    uint32_t     offset;
};

struct nvme_cmd_vendor_specific {
    uint32_t     buffNumDW;
    uint32_t     metaNumDW;
    uint32_t     vndrCDW12;
    uint32_t     vndrCDW13;
    uint32_t     vndrCDW14;
    uint32_t     vndrCDW15;
};

struct nvme_cmd {
    union {
        struct {
            struct nvme_cmd_hdr header;
            union {
                struct nvme_cmd_firmware_download firmwareDownload;
                struct nvme_cmd_firmware_activate firmwareActivate;
                struct nvme_cmd_get_log_page getLogPage;
                struct nvme_cmd_vendor_specific vendorSpecific;
                uint32_t    asUlong[6];
            } cmd;
        };
        uint32_t dw[16];
    };
};

struct cq_entry {
    union {
        uint32_t cmdSpecific;
        uint32_t numSubQAlloc:16, numCplQAlloc:16;
    } param;
    uint32_t reserved;
    uint32_t sqHdPtr:16, sqID:16;
    uint32_t cmdID:16, phaseTag:1, SC:8, SCT:3,:2, more:1, noRetry:1;
};

#endif  /* NVME_EXPRESS_H */
